"use client"
/* */
import { ItemCollection } from './item/ItemCollection.mjs';
import { BatchesController } from './pagination/BatchesController.mjs';
import { Filter } from './filter/Filter.mjs';
import { Sorting } from './sorting/Sorting.mjs';
import { ReactTable } from './ReactTable.mjs';
import { IncrementalLoader } from './loading/IncrementalLoader.mjs';
import invariant from 'invariant';
import { Search } from './search/Search.mjs';
import { ItemView } from './item/ItemView.mjs';
import { Table } from './table/Table.mjs';
import { useState, useEffect } from 'react';
import { useSettings } from '../../SettingsProvider/SettingsProvider.mjs';
import z from 'zod';

class List {
  static viewModeSettingsStorageSchema = z.enum(["list", "table", "tiles"]).optional();
  filters;
  itemView;
  table;
  search;
  sorting;
  items;
  reactTable;
  batches;
  loader;
  onAction;
  accordion;
  getItemId;
  componentProps;
  viewMode;
  setViewMode;
  supportsSettingsStorage;
  settingStorageKey;
  settingsStore;
  viewModeStorageKey;
  filterSettingsStorageKey;
  sortingStorageKey;
  constructor(shape) {
    const {
      settingStorageKey,
      itemView,
      table,
      filters = [],
      sorting = [],
      batchesController,
      onChange,
      loader,
      search,
      onAction,
      getItemId,
      defaultViewMode,
      accordion = false,
      ...componentProps
    } = shape;
    this.settingsStore = useSettings();
    this.settingStorageKey = settingStorageKey;
    this.filterSettingsStorageKey = settingStorageKey ? `${settingStorageKey}.activeFilters` : void 0;
    this.viewModeStorageKey = settingStorageKey ? `${settingStorageKey}.viewMode` : void 0;
    this.sortingStorageKey = settingStorageKey ? `${settingStorageKey}.sorting` : void 0;
    this.supportsSettingsStorage = !!this.settingStorageKey;
    this.items = new ItemCollection(this);
    this.filters = filters.map((shape2) => new Filter(this, shape2));
    this.sorting = sorting.map((shape2) => new Sorting(this, shape2));
    this.search = search ? new Search(this, search) : void 0;
    this.itemView = itemView ? new ItemView(this, itemView) : void 0;
    this.accordion = accordion;
    this.table = table ? new Table(this, table) : void 0;
    this.batches = new BatchesController(this, batchesController);
    this.componentProps = componentProps;
    this.loader = IncrementalLoader.useNew(this, loader);
    this.onAction = onAction;
    this.getItemId = getItemId;
    this.reactTable = ReactTable.useNew(this, onChange, {
      manualFiltering: this.loader.manualFiltering,
      manualPagination: this.loader.manualPagination,
      manualSorting: this.loader.manualSorting
    });
    const [viewMode, setViewMode] = useState(
      this.getStoredViewModeDefaultSetting() ?? defaultViewMode ?? "list"
    );
    this.viewMode = viewMode;
    this.setViewMode = (viewMode2) => {
      setViewMode(viewMode2);
      if (this.settingsStore && this.viewModeStorageKey) {
        this.settingsStore.set(
          "List",
          this.viewModeStorageKey,
          List.viewModeSettingsStorageSchema,
          viewMode2
        );
      }
    };
    useEffect(() => {
      this.filters.forEach((f) => f.deleteUnknownFilterValues());
    }, [this.filters]);
  }
  get isFiltered() {
    return this.filters.some((f) => f.isActive()) || !!this.search && this.search.isSet;
  }
  get visibleSorting() {
    return this.sorting.filter((s) => s.defaultEnabled !== "hidden");
  }
  static useNew(shape) {
    return new List(shape);
  }
  storeFilterDefaultSettings() {
    if (this.settingsStore && this.filterSettingsStorageKey) {
      const data = Object.fromEntries(
        this.filters.map((f) => [
          f.property,
          f.getArrayValue().filter((v) => v.isActive).map((v) => v.id)
        ])
      );
      this.settingsStore.set(
        "List",
        this.filterSettingsStorageKey,
        Filter.settingsStorageSchema,
        data
      );
    }
  }
  getStoredFilterDefaultSettings() {
    if (this.settingsStore && this.filterSettingsStorageKey) {
      return this.settingsStore.get(
        "List",
        this.filterSettingsStorageKey,
        Filter.settingsStorageSchema
      );
    }
  }
  getStoredViewModeDefaultSetting() {
    if (this.settingsStore && this.viewModeStorageKey) {
      return this.settingsStore.get(
        "List",
        this.viewModeStorageKey,
        List.viewModeSettingsStorageSchema
      );
    }
  }
  storeSortingSettings(sorting) {
    if (this.settingsStore && this.sortingStorageKey) {
      this.settingsStore.set(
        "List",
        this.sortingStorageKey,
        Sorting.storageSchema,
        { direction: sorting.direction, property: sorting.property }
      );
    }
  }
  getStoredSortingDefaultSetting() {
    if (this.settingsStore && this.sortingStorageKey) {
      return this.settingsStore.get(
        "List",
        this.sortingStorageKey,
        Sorting.storageSchema
      );
    }
  }
  getSorting(id) {
    const sorting = this.sorting.find((s) => s.id === id);
    invariant(!!sorting, `Could not get Sorting (ID: ${id})`);
    return sorting;
  }
  clearSorting() {
    return this.sorting.forEach((s) => s.clear());
  }
  resetFilters() {
    return this.filters.forEach((f) => f.resetValues());
  }
  clearFilters() {
    return this.filters.forEach((f) => f.clear());
  }
  useIsEmpty() {
    return !this.loader.useIsLoading() && this.items.entries.length === 0;
  }
}

export { List, List as default };
//# sourceMappingURL=List.mjs.map
